(***************************************************************************

Abstract syntax trees for JavaCard source files

VerifiCard Project - Dmons research team - LRI - Universit Paris XI

$Id: ast.mli,v 1.16 2003/10/02 06:58:19 marche Exp $

***************************************************************************)

open Ast_types;;

val mk_expr : Location.t -> expr_node -> expr;;
val mk_expr_no_loc : expr_node -> expr;;

val mk_lit : Location.t -> literal -> expr;;

val mk_un : Location.t -> string -> expr -> expr;;

val mk_prepost : Location.t -> incrdecrop -> expr -> expr;;

val mk_bin : Location.t -> expr -> string -> expr -> expr;;

val expr_true : expr;;
val expr_false : expr;;
val expr_zero : expr;;

val expand_name : qualified_ident -> expr;;

val build_array_creation_expr : 
  type_expr -> expr list * int -> array_creation;;

(*s JML annotations *)

val fprintf_assignable_clause : Format.formatter -> assignable_clause -> unit;;

val default_loop_annotation : loop_annotation;;

val fprintf_loop_annotation : Format.formatter -> loop_annotation -> unit;;
     
val default_spec_header : spec_header;;
 
val concat_specs : 
    method_specification -> method_specification -> method_specification;;

val default_method_specification : method_specification;;

exception Not_a_statement_spec of string;;
val to_statement_spec : method_specification -> statement_specification;;

val fprintf_method_specification :
  Format.formatter -> method_specification -> unit;;

val default_jml_declaration : jml_declaration;;

val default_jml_specification : jml_specification

(*s statements *)

val make_block : block -> statement

(*s printing functions *)

(* be careful, these functions use [Format] module *)

val fprintf_expr : Format.formatter -> expr -> unit

val fprintf_statement : Format.formatter -> statement -> unit

val fprintf_field_declaration : Format.formatter -> field_declaration -> unit

val printf_compilation_unit : compilation_unit -> unit
