
Open Scope Z_scope.

Notation " this # f " := (acc f this) (at level 30, f at level 0).
Notation " heap : this [ i ] " := (array_acc heap this i)
  (at level 30, this at level 0, i at level 50).


Ltac Simpl1Hyps :=
  match goal with
  | id1:?X1,id2:?X1 |- _ => clear id2
  | id:True |- _ => clear id
  | id:(?X1 = ?X1) |- _ => clear id
  | id:(?X1 /\ ?X2) |- _ =>
      decompose record id; clear id
  | id:(store_extends ?X1 ?X1) |- _ =>
      clear id
  | id:(modifiable ?X2 ?X1 ?X1 ?X3) |- _ =>
      clear id
  | id:(array_modifiable ?X2 ?X1 ?X1 ?X3) |- _ => clear id
  | _ => fail
  end.

Ltac SimplHyps := repeat Simpl1Hyps.


Ltac Acc_update1 :=
  rewrite acc_update ||
    rewrite array_acc_update ||
      (rewrite acc_update_eq; [ idtac | progress auto with * ]) ||
        (rewrite array_acc_update_eq;
          [ idtac | progress auto with * | progress auto with * ]) ||
          (rewrite acc_update_other; [ idtac | progress auto with * ]) ||
            (rewrite array_acc_update_other;
              [ idtac | progress auto with * ]); auto with *.

Ltac krakatoa := repeat Acc_update1.

Ltac CleanMod :=
  match goal with
  | id:(store_extends ?X1 ?X1) |- _ =>
      clear id; CleanMod
  | id:(modifiable ?X2 ?X1 ?X1 ?X3) |- _ =>
      clear id; CleanMod
  | id:(array_modifiable ?X2 ?X1 ?X1 ?X3) |- _ => clear id; CleanMod
  | _ => idtac
  end.

Ltac Subtype :=
  match goal with
  |  |- (subType ?X1 ?X2) => exact (dec_A_proof (subType_dec X1 X2) I)
  end.

Ltac NotSubtype :=
  match goal with
  |  |- (~ subType ?X1 ?X2) =>
      exact (dec_not_A_proof (subType_dec X1 X2) I)
  end.

(*
  |  |- (extends ?X1 ?X2) => exact (dec_A_proof (extends_dec X1 X2) I)
  |  |- (~ extends ?X1 ?X2) =>
      exact (dec_not_A_proof (extends_dec X1 X2) I)
  end.
*)


Ltac Instance :=
  match goal with
  | id:(instanceof ?X1 ?X2 ?X3) |- (instanceof ?X1 ?X2 ?X3) =>
      exact id
  | id:(typeof ?X1 ?X2 ?X3) |- (instanceof ?X1 ?X2 ?X3) =>
      apply typeof_instanceof
  | id:(instanceof ?X1 ?X2 ?X3) |- (instanceof ?X4 ?X2 ?X3) =>
      apply (instanceof_extends_stable (h:=X1) (h':=X4) (t:=X3) (v:=X2) id)
  | id:(instanceof ?X1 ?X2 ?X3) |- (instanceof ?X4 ?X2 ?X5) =>
      apply (instanceof_subType_trans (h:=X4) (this:=X2) (t1:=X3));
       [ Instance | Subtype ]
  | id:(typeof ?X1 ?X2 ?X3) |- (instanceof ?X1 ?X2 ?X3) =>
     apply (typeof_instanceof id)
  | id:(typeof ?X1 ?X2 ?X3) |- (instanceof ?X1 ?X2 ?X4) =>
     apply (typeof_subtype_instanceof (t1:=X3) (t2:=X4) id); Subtype
  | _ => fail
  end.

Ltac StoreRec :=
  match goal with
  | id:(store_extends ?X1 ?X2) |- (store_extends ?X1 ?X2) => 
  exact id
  |  |- (store_extends ?X1 ?X1) =>
      exact (store_extends_refl (t:=X1))
  | id:(store_extends ?X1 ?X2) |- (store_extends ?X1 ?X3) =>
      apply (store_extends_trans (t1:=X1) (t2:=X2) (t3:=X3) id); StoreRec
  | id:(store_extends ?X2 ?X3) |- (store_extends ?X1 ?X3) =>
      apply (store_extends_trans (t1:=X1) (t2:=X2) (t3:=X3));
       [ StoreRec | exact id ]
  | _ => fail
  end.

Ltac Store := CleanMod; StoreRec.

Ltac AliveRec :=
  match goal with
  | id:(alive ?X1 ?X2) |- (alive ?X1 ?X2) => exact id
  |  |- (alive ?X1 Null) =>
      apply null_alive; reflexivity
  | id:(instanceof ?X1 ?X2 ?X3) |- (alive ?X1 ?X2) =>
      apply
       (instanceof_alive (h:=X1) (t:=X3) (v:=X2) id)
  | id:(typeof ?X1 ?X2 ?X3) |- (alive ?X1 ?X2) =>
      apply
       (typeof_alive (h:=X1) (t:=X3) (v:=X2) id)
  | id:(?X1 = ?X2) |- (alive ?X1 ?X4) =>
      rewrite id; AliveRec
  | id:(store_extends ?X1 ?X2) |- (alive ?X2 ?X3) =>
      apply (alive_extends_stable (t:=X1) (t':=X2) id (v:=X3)); AliveRec
  | _ => fail
  end.

Ltac Alive := CleanMod; AliveRec.

Ltac Fresh :=
  match goal with
  | h1:(fresh ?X1 ?X2) |- (fresh ?X1 ?X2) => exact h1
  | h1:(?X2 = ?X3) |- (fresh ?X1 ?X2) =>
      rewrite h1; Fresh
  | h1:(fresh ?X3 ?X2) |- (fresh ?X1 ?X2) =>
      apply (fresh_extends_stable (t:=X1) (t':=X3)); progress auto
  | h1:(store_extends ?X1 ?X3) |- (fresh ?X1 ?X2) =>
      apply (fresh_extends_stable (t:=X1) (t':=X3) h1); Fresh
  | _ => fail
  end.

Ltac Neqv :=
  match goal with
  | id:(fresh ?X1 ?X2) |- (?X2 != Null) =>
      apply (fresh_not_null (h:=X1) (v:=X2) id)
  | id:(fresh ?X1 ?X2) |- (?X2 != ?X3) =>
      apply (fresh_alive_diff (t:=X1) (v1:=X2) (v2:=X3) id); Alive
  | id:(fresh ?X1 ?X2) |- (?X3 != ?X2) =>
      apply (alive_fresh_diff (t:=X1) (v1:=X3) (v2:=X2));
       [ Alive | exact id ]
  | id:(?X3 = arraylength ?X1 ?X2) |- (?X2 != Null) =>
      apply (arraylength_pos_val_not_null (h:=X1) (v:=X2));
       progress auto with zarith
  | id:(arraylength ?X1 ?X2 = ?X3) |- (?X2 != Null) =>
      apply (arraylength_pos_val_not_null (h:=X1) (v:=X2));
       progress auto with zarith
  | id:(?X3 <= arraylength ?X1 ?X2) |- (?X2 != Null) =>
      apply (arraylength_pos_val_not_null (h:=X1) (v:=X2));
       progress auto with zarith
  | id:(?X3 < arraylength ?X1 ?X2) |- (?X2 != Null) =>
      apply (arraylength_pos_val_not_null (h:=X1) (v:=X2));
       progress auto with zarith
  | id:(arraylength ?X1 ?X2 >= ?X3) |- (?X2 != Null) =>
      apply (arraylength_pos_val_not_null (h:=X1) (v:=X2));
       progress auto with zarith
  | id:(arraylength ?X1 ?X2 > ?X3) |- (?X2 != Null) =>
      apply (arraylength_pos_val_not_null (h:=X1) (v:=X2));
       progress auto with zarith
  |  |- (Null != ?X2) => apply neqv_sym; Neqv
  | _ => fail
  end.


Ltac ModRec :=
  match goal with
  | id:(modifiable ?X1 ?X2 ?X3 ?X4) |- (modifiable ?X1 ?X2 ?X3 ?X4) =>
      exact id
  |  |- (modifiable ?X1 ?X2 ?X2 ?X4) =>
      exact (modifiable_refl (h:=X1) X2 (unchanged:=X4))
  | id:(fresh ?X1 ?X3) |- (modifiable ?X1 ?X7 (update ?X2 ?X3 ?X4) ?X6) =>
      apply
       (modifiable_fresh_update (h:=X1) (m1:=X7) (m2:=X2) (unch:=X6) (v:=X3)
          X4 id); ModRec
  |  |- (modifiable ?X1 ?X2 (update ?X2 ?X3 ?X4) ?X6) =>
      apply (modifiable_update (h:=X1) X2 (unch:=X6) (v:=X3) X4); subst;
       krakatoa; progress auto
  |  |- (modifiable ?X1 ?X7 (update ?X2 ?X3 ?X4) ?X6) =>
      apply
       (modifiable_trans (h:=X1) (m1:=X7) (m2:=X2) (m3:=(
          update X2 X3 X4)) (unch:=X6)); ModRec
  | id:(?X2 = ?X3) |- (modifiable ?X1 ?X4 ?X2 ?X5) =>
      rewrite id; ModRec
  | id:(modifiable ?X1 ?X2 ?X3 ?X4) |- (modifiable ?X5 ?X6 ?X3 ?X7) =>
      apply modifiable_trans_sub with (4:=id);
       [ subst; krakatoa; unfold inter_loc; intuition; fail 
       | Store | ModRec ]
  end.


Ltac Modifiable := CleanMod; ModRec.

Ltac ArrayModRec :=
  match goal with
  | id:(array_modifiable ?X1 ?X2 ?X3 ?X4) |-
  (array_modifiable ?X1 ?X2 ?X3 ?X4) => exact id
  |  |- (array_modifiable ?X1 ?X2 ?X2 ?X4) =>
      exact (array_modifiable_refl (h:=X1) X2 (unchanged:=X4))
  | id:(fresh ?X1 ?X3) |-
  (array_modifiable ?X1 ?X7 (array_update ?X2 ?X3 ?X4 ?X8) ?X6) =>
      apply
       (array_modifiable_fresh_update (h:=X1) (m1:=X7) (m2:=X2) (unch:=X6)
          (v:=X3) X4 X8 id); ArrayModRec
  |  |- (array_modifiable ?X1 ?X2 (array_update ?X2 ?X3 ?X4 ?X7) ?X6) =>
      apply
       (array_modifiable_update (h:=X1) X2 (unch:=X6) (v:=X3) (i:=X4) X7);
       subst; krakatoa; progress auto
  |  |- (array_modifiable ?X1 ?X7 (array_update ?X2 ?X3 ?X4 ?X8) ?X6) =>
      apply
       (array_modifiable_trans (h:=X1) (m1:=X7) (m2:=X2)
          (m3:=(array_update X2 X3 X4 X8)) (unch:=X6));
       [ ArrayModRec
       | apply
          (array_modifiable_update (h:=X1) X2 (unch:=X6) (v:=X3) (i:=X4) X8);
          progress auto ]
  | id:(?X2 = ?X3) |- (array_modifiable ?X1 ?X4 ?X2 ?X5) =>
      rewrite id; ArrayModRec
  | id:(array_modifiable ?X1 ?X2 ?X3 ?X4) |-
  (array_modifiable ?X5 ?X6 ?X3 ?X7) =>
      apply
       (array_modifiable_trans_sub (h1:=X5) (h2:=X1) (m1:=X6) (m2:=X2)
          (m3:=X3) (unch1:=X7) (unch2:=X4));
       [ subst; krakatoa; unfold array_inter_loc; intuition; fail 
       | Store | ArrayModRec | exact id ]
  end.


Ltac ArrayModifiable := CleanMod; ArrayModRec.


Hint Extern 0 (alive _ _) => Alive.
Hint Extern 0 (fresh _ _) => Fresh.
Hint Extern 0 (store_extends _ _) => Store.
Hint Extern 0 (instanceof _ _ _) => Instance.
Hint Extern 0 (modifiable _ _ _ _) => Modifiable.
Hint Extern 0 (_ != _) => Neqv.
Hint Extern 0 (array_modifiable _ _ _ _) => ArrayModifiable.
Hint Extern 0 (subType _ _) => Subtype.
Hint Extern 0 (~(subType _ _)) => NotSubtype.
