
Require Import preambule.
Section Map.

Set Implicit Arguments.
Unset Strict Implicit.

Variables I V : Set.
Variable eq_I_dec : forall i1 i2:I, {i1 = i2} + {i1 <> i2}.

Definition t := I -> option V.

Definition cte (v:V) : t := fun i => None.

Definition find (x:t) : I -> option V := x.
Definition add (m:t) (i:I) (v:V) : t :=
  fun i' => if eq_I_dec i i' then fun _ => Some v else fun _ => m i'.

Lemma find_add : forall (m:t) (i:I) (v:V), find (add m i v) i = Some v.
Proof.
unfold find, add in |- *; intros.
case (eq_I_dec i i); intros; auto.
case n; trivial.
Qed.

Lemma find_add_eq :
 forall (m:t) (i i':I) (v:V), i = i' -> find (add m i v) i' = Some v.
Proof.
simple destruct 1; apply find_add.
Qed.

Lemma find_add_other :
 forall (m:t) (i i':I) (v:V), i <> i' -> find (add m i v) i' = m i'.
Proof.
unfold find, add in |- *; intros.
case (eq_I_dec i i'); intros; auto.
case H; trivial.
Qed.

Definition contains (m:t) (i:I) (v:V) : Prop := m i = Some v.

Lemma contains_find :
 forall (m:t) (i:I) (v:V), contains m i v -> find m i = Some v.
trivial.
Qed.

Lemma find_contains :
 forall (m:t) (i:I) (v:V), find m i = Some v -> contains m i v.
trivial.
Qed.

Lemma contains_add :
 forall (m:t) (i i':I) (v:V), i = i' -> contains (add m i v) i' v.
intros; apply find_contains; auto.
apply find_add_eq; auto.
Qed.

Lemma contains_add_other :
 forall (m:t) (i i':I) (v:V),
   i <> i' -> contains m i' v -> contains (add m i v) i' v.
intros; apply find_contains; auto.
rewrite find_add_other; auto.
Qed.

End Map.
Implicit Arguments cte [V].