# 12 "java_lexer.mll"
 

  open Java_parser

  exception Lexical_error of string

  let string_buffer = Buffer.create 17

  let kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ "abstract", ABSTRACT;
	  "boolean", BOOLEAN;
	  "break", BREAK;
	  "byte", BYTE;
	  "byvalue", BYVALUE;
	  "case", CASE;
	  "cast", CAST;
	  "catch", CATCH;
	  "char", CHAR;
	  "class", CLASS;
	  "const", CONST;
	  "continue", CONTINUE;
	  "default", DEFAULT;
	  "do", DO;
	  "double", DOUBLE;
	  "else", ELSE;
	  "extends", EXTENDS;
	  "false", FALSE(Location.dummy);
	  "final", FINAL;
	  "finally", FINALLY;
	  "float", FLOAT;
	  "for", FOR;
	  "future", FUTURE;
	  "generic", GENERIC;
	  "goto", GOTO;
	  "if", IF;
	  "implements", IMPLEMENTS;
	  "import", IMPORT;
	  "inner", INNER;
	  "instanceof", INSTANCEOF;
	  "int", INT;
	  "interface", INTERFACE;
	  "long", LONG;
	  "native", NATIVE;
	  "new", NEW(Location.dummy);
	  "null", NULL(Location.dummy);
	  "operator", OPERATOR;
	  "outer", OUTER;
	  "package", PACKAGE;
	  "private", PRIVATE;
	  "protected", PROTECTED;
	  "public", PUBLIC;
	  "rest", REST;
	  "return", RETURN;
	  "short", SHORT;
	  "static", STATIC;
	  "super", SUPER(Location.dummy);
	  "switch", SWITCH;
	  "synchronized", SYNCHRONIZED;
	  "this", THIS(Location.dummy);
	  (* "threadsafe" ? *)
	  "throw", THROW;
	  "throws", THROWS;
	  "transient", TRANSIENT;
	  "true", TRUE(Location.dummy);
	  "try", TRY;
	  "var", VAR;
	  "void", VOID;
	  "volatile", VOLATILE;
	  "while", WHILE;	
	]
    in table

  let find_loc lexbuf =
    Location.find 
      !Java_parser_base.cur_base 
      (Lexing.lexeme_start lexbuf) 
      (Lexing.lexeme_end lexbuf) 

  let id_or_kw lexbuf s =
    let l = find_loc lexbuf in
    try
      let k = Hashtbl.find kw_table s in
      (*i
	prerr_string "Keyword ";
	prerr_endline s;
	i*)
      begin
	match k with
	  | FALSE _ -> FALSE(l)
	  | NEW _ -> NEW(l)
	  | NULL _ -> NULL(l)
	  | SUPER _ -> SUPER(l)
	  | THIS _ -> THIS(l)
	  | TRUE _ -> TRUE(l)
	  | _ -> k
      end
    with
	Not_found ->
	  (*i
	    prerr_string "Ident ";
	    prerr_endline s;
	    i*)
	  ID(l,s)


  let remove_last_char s = String.sub s 0 (pred (String.length s))

  let jml_spec_token base jml_string =
(*i*)
    Format.fprintf Config.log "In file %s, parsing JML Spec: %s@."
      (Location.base_filename base) jml_string;
(*i*)
    match Jml_syntax.parse_jml_specification base jml_string with 
    | Ast_types.Jml_pure -> JML_PURE
    | Ast_types.Jml_spec_public -> JML_SPEC_PUBLIC
    | Ast_types.Jml_declaration d -> JML_DECLARATIONS(d)
    | Ast_types.Jml_method_specification s -> JML_METHOD_SPECIFICATION(s)
    | Ast_types.Jml_loop_annotation la -> JML_LOOP_ANNOTATION(la)
    | Ast_types.Jml_assertion a -> JML_ASSERTION(a)


# 128 "java_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\201\255\003\000\254\255\079\000\246\255\245\255\231\255\
    \229\255\228\255\003\000\160\000\031\000\161\000\158\000\167\000\
    \035\000\082\000\077\000\078\000\080\000\081\000\160\000\181\000\
    \216\000\002\000\208\255\207\255\206\255\205\255\204\255\203\255\
    \202\255\200\255\131\000\209\255\221\255\245\000\044\001\069\001\
    \104\001\021\001\210\255\191\000\079\001\211\255\219\255\139\001\
    \215\255\054\001\165\001\175\001\217\255\236\001\204\001\213\255\
    \246\001\003\002\223\255\110\000\224\255\225\255\112\000\114\000\
    \233\255\234\255\240\255\241\255\210\000\133\001\250\255\003\000\
    \248\255\171\000\251\255\253\255\051\002\255\255\230\001\252\255\
    \106\001\176\001\010\001\173\000\025\001\141\001\176\000\157\001\
    \158\001\177\000\165\001\052\002\213\000\178\000\053\002\249\255\
    \005\000\007\000\019\000";
  Lexing.lex_backtrk = 
   "\255\255\255\255\000\000\255\255\008\000\255\255\255\255\255\255\
    \255\255\255\255\016\000\017\000\018\000\012\000\013\000\019\000\
    \025\000\020\000\031\000\031\000\029\000\023\000\011\000\033\000\
    \033\000\054\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\039\000\255\255\
    \035\000\255\255\255\255\255\255\043\000\255\255\255\255\255\255\
    \255\255\255\255\039\000\037\000\255\255\041\000\255\255\255\255\
    \255\255\041\000\255\255\028\000\255\255\255\255\028\000\028\000\
    \255\255\255\255\255\255\255\255\006\000\255\255\255\255\255\255\
    \255\255\003\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \004\000\003\000\255\255\003\000\002\000\255\255\003\000\002\000\
    \255\255\003\000\002\000\255\255\005\000\005\000\004\000\255\255\
    \255\255\255\255\001\000";
  Lexing.lex_default = 
   "\001\000\000\000\255\255\000\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\034\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\000\000\255\255\255\255\000\000\000\000\255\255\
    \000\000\255\255\255\255\255\255\000\000\255\255\255\255\000\000\
    \255\255\255\255\000\000\255\255\000\000\000\000\255\255\255\255\
    \000\000\000\000\000\000\000\000\255\255\071\000\000\000\071\000\
    \000\000\255\255\000\000\000\000\080\000\000\000\075\000\000\000\
    \080\000\080\000\084\000\255\255\084\000\087\000\255\255\087\000\
    \090\000\255\255\090\000\094\000\255\255\255\255\094\000\000\000\
    \096\000\098\000\098\000";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\003\000\000\000\002\000\072\000\000\000\003\000\
    \000\000\077\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\000\000\000\000\
    \002\000\021\000\032\000\002\000\000\000\012\000\015\000\025\000\
    \026\000\027\000\010\000\013\000\006\000\014\000\022\000\011\000\
    \024\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\009\000\005\000\018\000\020\000\019\000\008\000\
    \061\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\030\000\061\000\031\000\016\000\004\000\
    \061\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\028\000\017\000\029\000\007\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\063\000\060\000\060\000\059\000\058\000\058\000\061\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\035\000\061\000\062\000\061\000\004\000\061\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\068\000\066\000\067\000\065\000\064\000\069\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\075\000\061\000\077\000\061\000\061\000\077\000\
    \077\000\075\000\000\000\038\000\061\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\042\000\041\000\042\000\073\000\000\000\000\000\077\000\
    \033\000\036\000\255\255\255\255\096\000\255\255\038\000\075\000\
    \040\000\040\000\040\000\040\000\040\000\040\000\040\000\040\000\
    \039\000\039\000\074\000\255\255\003\000\000\000\000\000\000\000\
    \000\000\042\000\041\000\042\000\042\000\041\000\042\000\000\000\
    \000\000\036\000\000\000\255\255\036\000\051\000\051\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\000\000\
    \037\000\000\000\000\000\000\000\083\000\000\000\051\000\051\000\
    \051\000\051\000\051\000\051\000\042\000\041\000\042\000\000\000\
    \043\000\000\000\043\000\255\255\036\000\044\000\044\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\044\000\044\000\000\000\
    \037\000\000\000\000\000\000\000\000\000\000\000\051\000\051\000\
    \051\000\051\000\051\000\051\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\050\000\050\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\050\000\050\000\
    \048\000\047\000\048\000\038\000\255\255\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\042\000\041\000\042\000\255\255\000\000\000\000\072\000\
    \048\000\047\000\048\000\045\000\000\000\045\000\038\000\003\000\
    \040\000\040\000\040\000\040\000\040\000\040\000\040\000\040\000\
    \039\000\039\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \003\000\042\000\041\000\042\000\042\000\041\000\042\000\255\255\
    \000\000\000\000\000\000\045\000\046\000\045\000\049\000\086\000\
    \049\000\000\000\255\255\050\000\050\000\050\000\050\000\050\000\
    \050\000\050\000\050\000\050\000\050\000\070\000\255\255\255\255\
    \089\000\000\000\000\000\000\000\042\000\041\000\042\000\255\255\
    \000\000\000\000\255\255\000\000\046\000\050\000\050\000\050\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\050\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\048\000\000\000\048\000\000\000\000\000\000\000\000\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\000\000\056\000\
    \000\000\056\000\000\000\052\000\057\000\057\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\057\000\000\000\000\000\
    \003\000\048\000\074\000\048\000\255\255\003\000\000\000\000\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\000\000\000\000\
    \000\000\255\255\000\000\052\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\057\000\057\000\057\000\
    \055\000\054\000\055\000\057\000\057\000\057\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\079\000\079\000\255\255\
    \081\000\000\000\003\000\000\000\000\000\000\000\000\000\055\000\
    \003\000\055\000\000\000\000\000\003\000\000\000\000\000\000\000\
    \055\000\054\000\055\000\000\000\003\000\077\000\000\000\000\000\
    \003\000\000\000\003\000\000\000\000\000\000\000\093\000\255\255\
    \000\000\000\000\000\000\092\000\255\255\000\000\000\000\055\000\
    \000\000\055\000\255\255\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\070\000\255\255\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\074\000\000\000\078\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\074\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\070\000\095\000\255\255";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\255\255\002\000\071\000\255\255\096\000\
    \255\255\097\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\098\000\255\255\255\255\
    \000\000\000\000\000\000\002\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\012\000\000\000\000\000\000\000\
    \016\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\018\000\018\000\019\000\019\000\020\000\021\000\017\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\034\000\059\000\059\000\062\000\004\000\063\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\011\000\014\000\013\000\015\000\017\000\011\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\073\000\014\000\083\000\011\000\013\000\086\000\
    \089\000\093\000\255\255\023\000\015\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\023\000\023\000\023\000\068\000\255\255\255\255\092\000\
    \000\000\023\000\025\000\071\000\092\000\096\000\024\000\097\000\
    \024\000\024\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\068\000\098\000\082\000\255\255\255\255\255\255\
    \255\255\023\000\023\000\023\000\024\000\024\000\024\000\255\255\
    \255\255\023\000\255\255\084\000\024\000\037\000\037\000\037\000\
    \037\000\037\000\037\000\037\000\037\000\037\000\037\000\255\255\
    \024\000\255\255\255\255\255\255\082\000\255\255\037\000\037\000\
    \037\000\037\000\037\000\037\000\024\000\024\000\024\000\255\255\
    \041\000\255\255\041\000\084\000\024\000\041\000\041\000\041\000\
    \041\000\041\000\041\000\041\000\041\000\041\000\041\000\255\255\
    \024\000\255\255\255\255\255\255\255\255\255\255\037\000\037\000\
    \037\000\037\000\037\000\037\000\038\000\038\000\038\000\038\000\
    \038\000\038\000\038\000\038\000\038\000\038\000\049\000\049\000\
    \049\000\049\000\049\000\049\000\049\000\049\000\049\000\049\000\
    \038\000\038\000\038\000\039\000\080\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\044\000\
    \044\000\044\000\044\000\044\000\044\000\044\000\044\000\044\000\
    \044\000\039\000\039\000\039\000\080\000\255\255\255\255\069\000\
    \038\000\038\000\038\000\044\000\255\255\044\000\040\000\085\000\
    \040\000\040\000\040\000\040\000\040\000\040\000\040\000\040\000\
    \040\000\040\000\255\255\255\255\255\255\255\255\255\255\087\000\
    \088\000\039\000\039\000\039\000\040\000\040\000\040\000\090\000\
    \255\255\255\255\255\255\044\000\040\000\044\000\047\000\085\000\
    \047\000\255\255\081\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\047\000\047\000\047\000\069\000\080\000\087\000\
    \088\000\255\255\255\255\255\255\040\000\040\000\040\000\090\000\
    \255\255\255\255\081\000\255\255\040\000\050\000\050\000\050\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\050\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\050\000\255\255\050\000\255\255\255\255\255\255\255\255\
    \051\000\051\000\051\000\051\000\051\000\051\000\255\255\054\000\
    \255\255\054\000\255\255\051\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\255\255\255\255\
    \078\000\050\000\082\000\050\000\081\000\078\000\255\255\255\255\
    \051\000\051\000\051\000\051\000\051\000\051\000\255\255\255\255\
    \255\255\084\000\255\255\051\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \053\000\053\000\053\000\057\000\057\000\057\000\057\000\057\000\
    \057\000\057\000\057\000\057\000\057\000\076\000\091\000\094\000\
    \076\000\255\255\078\000\255\255\255\255\255\255\255\255\057\000\
    \078\000\057\000\255\255\255\255\078\000\255\255\255\255\255\255\
    \053\000\053\000\053\000\255\255\078\000\076\000\255\255\255\255\
    \078\000\255\255\078\000\255\255\255\255\255\255\091\000\094\000\
    \255\255\255\255\255\255\091\000\094\000\255\255\255\255\057\000\
    \255\255\057\000\080\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\091\000\094\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\069\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\085\000\255\255\076\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\087\000\088\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\090\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \081\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\078\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\076\000\091\000\094\000";
  Lexing.lex_base_code = 
   "";
  Lexing.lex_backtrk_code = 
   "";
  Lexing.lex_default_code = 
   "";
  Lexing.lex_trans_code = 
   "";
  Lexing.lex_check_code = 
   "";
  Lexing.lex_code = 
   "";
}

let rec token lexbuf =
    __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 140 "java_lexer.mll"
      ( token lexbuf )
# 401 "java_lexer.ml"

  | 1 ->
# 142 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf; token lexbuf )
# 406 "java_lexer.ml"

  | 2 ->
# 144 "java_lexer.mll"
      ( token lexbuf )
# 411 "java_lexer.ml"

  | 3 ->
# 146 "java_lexer.mll"
      ( Buffer.clear string_buffer;
	DOC_COMMENT(doc_comment lexbuf) )
# 417 "java_lexer.ml"

  | 4 ->
# 149 "java_lexer.mll"
      ( Buffer.clear string_buffer;
	let base = 
	  Location.offset 
	    !Java_parser_base.cur_base (Lexing.lexeme_end lexbuf) 
	in
	jml_spec_token base (jml_spec lexbuf) )
# 427 "java_lexer.ml"

  | 5 ->
# 156 "java_lexer.mll"
      ( Buffer.clear string_buffer;
	let base = 
	  Location.offset 
	    !Java_parser_base.cur_base (Lexing.lexeme_end lexbuf) 
	in
	jml_spec_token base (jml_line_spec lexbuf) )
# 437 "java_lexer.ml"

  | 6 ->
# 163 "java_lexer.mll"
      ( comment lexbuf; token lexbuf )
# 442 "java_lexer.ml"

  | 7 ->
# 165 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf; token lexbuf )
# 447 "java_lexer.ml"

  | 8 ->
# 167 "java_lexer.mll"
      ( id_or_kw lexbuf (Lexing.lexeme lexbuf) )
# 452 "java_lexer.ml"

  | 9 ->
# 169 "java_lexer.mll"
      ( SEMICOLON )
# 457 "java_lexer.ml"

  | 10 ->
# 171 "java_lexer.mll"
      ( COMMA )
# 462 "java_lexer.ml"

  | 11 ->
# 173 "java_lexer.mll"
      ( DOT )
# 467 "java_lexer.ml"

  | 12 ->
# 175 "java_lexer.mll"
      ( PLUS(find_loc lexbuf) )
# 472 "java_lexer.ml"

  | 13 ->
# 177 "java_lexer.mll"
      ( MINUS(find_loc lexbuf) )
# 477 "java_lexer.ml"

  | 14 ->
# 179 "java_lexer.mll"
      ( PLUSPLUS(find_loc lexbuf) )
# 482 "java_lexer.ml"

  | 15 ->
# 181 "java_lexer.mll"
      ( MINUSMINUS(find_loc lexbuf) )
# 487 "java_lexer.ml"

  | 16 ->
# 183 "java_lexer.mll"
      ( STAR )
# 492 "java_lexer.ml"

  | 17 ->
# 185 "java_lexer.mll"
      ( SLASH )
# 497 "java_lexer.ml"

  | 18 ->
# 187 "java_lexer.mll"
      ( PERCENT )
# 502 "java_lexer.ml"

  | 19 ->
# 189 "java_lexer.mll"
      ( AMPERSAND )
# 507 "java_lexer.ml"

  | 20 ->
# 191 "java_lexer.mll"
      ( VERTICALBAR )
# 512 "java_lexer.ml"

  | 21 ->
# 193 "java_lexer.mll"
      ( AMPERSANDAMPERSAND )
# 517 "java_lexer.ml"

  | 22 ->
# 195 "java_lexer.mll"
      ( VERTICALBARVERTICALBAR )
# 522 "java_lexer.ml"

  | 23 ->
# 197 "java_lexer.mll"
      ( BANG(find_loc lexbuf) )
# 527 "java_lexer.ml"

  | 24 ->
# 199 "java_lexer.mll"
      ( TILDA(find_loc lexbuf) )
# 532 "java_lexer.ml"

  | 25 ->
# 201 "java_lexer.mll"
      ( CARET )
# 537 "java_lexer.ml"

  | 26 ->
# 203 "java_lexer.mll"
      ( QUESTIONMARK )
# 542 "java_lexer.ml"

  | 27 ->
# 205 "java_lexer.mll"
      ( COLON )
# 547 "java_lexer.ml"

  | 28 ->
# 207 "java_lexer.mll"
      ( SHIFT(Lexing.lexeme lexbuf) )
# 552 "java_lexer.ml"

  | 29 ->
# 209 "java_lexer.mll"
      ( EQ )
# 557 "java_lexer.ml"

  | 30 ->
# 212 "java_lexer.mll"
      ( ASSIGNOP(Lexing.lexeme lexbuf) )
# 562 "java_lexer.ml"

  | 31 ->
# 214 "java_lexer.mll"
      ( COMP(Lexing.lexeme lexbuf) )
# 567 "java_lexer.ml"

  | 32 ->
# 216 "java_lexer.mll"
      ( EQOP(Lexing.lexeme lexbuf) )
# 572 "java_lexer.ml"

  | 33 ->
# 221 "java_lexer.mll"
      ( INTEGER(find_loc lexbuf,int_of_string (Lexing.lexeme lexbuf)) )
# 577 "java_lexer.ml"

  | 34 ->
# 223 "java_lexer.mll"
      ( INTEGER 
	  (find_loc lexbuf,
	   int_of_string (remove_last_char (Lexing.lexeme lexbuf))) )
# 584 "java_lexer.ml"

  | 35 ->
# 230 "java_lexer.mll"
      ( INTEGER
	  (find_loc lexbuf,
	   int_of_string ("0o" ^ (Lexing.lexeme lexbuf))) )
# 591 "java_lexer.ml"

  | 36 ->
# 234 "java_lexer.mll"
      ( INTEGER 
	  (find_loc lexbuf,
	   int_of_string 
	     ("0o" ^ (remove_last_char (Lexing.lexeme lexbuf)))) )
# 599 "java_lexer.ml"

  | 37 ->
# 242 "java_lexer.mll"
    ( INTEGER(find_loc lexbuf,int_of_string (Lexing.lexeme lexbuf)) )
# 604 "java_lexer.ml"

  | 38 ->
# 244 "java_lexer.mll"
    ( INTEGER
	(find_loc lexbuf,
	 int_of_string (remove_last_char (Lexing.lexeme lexbuf))) )
# 611 "java_lexer.ml"

  | 39 ->
# 251 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string  (Lexing.lexeme lexbuf)) )
# 618 "java_lexer.ml"

  | 40 ->
# 255 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string  (remove_last_char (Lexing.lexeme lexbuf))) )
# 625 "java_lexer.ml"

  | 41 ->
# 259 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string (Lexing.lexeme lexbuf)) )
# 632 "java_lexer.ml"

  | 42 ->
# 263 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string  (remove_last_char (Lexing.lexeme lexbuf))) )
# 639 "java_lexer.ml"

  | 43 ->
# 267 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string  (Lexing.lexeme lexbuf)) )
# 646 "java_lexer.ml"

  | 44 ->
# 271 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string  (remove_last_char (Lexing.lexeme lexbuf))) )
# 653 "java_lexer.ml"

  | 45 ->
# 275 "java_lexer.mll"
      ( REAL
	  (find_loc lexbuf,
	   float_of_string (remove_last_char (Lexing.lexeme lexbuf))) )
# 660 "java_lexer.ml"

  | 46 ->
# 282 "java_lexer.mll"
      ( CHARACTER
	  (find_loc lexbuf,
	   Lexing.lexeme_char lexbuf 1) )
# 667 "java_lexer.ml"

  | 47 ->
# 287 "java_lexer.mll"
      ( LEFTPAR(find_loc lexbuf) )
# 672 "java_lexer.ml"

  | 48 ->
# 289 "java_lexer.mll"
      ( RIGHTPAR )
# 677 "java_lexer.ml"

  | 49 ->
# 291 "java_lexer.mll"
      ( LEFTBRACE )
# 682 "java_lexer.ml"

  | 50 ->
# 293 "java_lexer.mll"
      ( RIGHTBRACE )
# 687 "java_lexer.ml"

  | 51 ->
# 295 "java_lexer.mll"
      ( LEFTBRACKET )
# 692 "java_lexer.ml"

  | 52 ->
# 297 "java_lexer.mll"
      ( RIGHTBRACKET )
# 697 "java_lexer.ml"

  | 53 ->
# 299 "java_lexer.mll"
      ( let start = Lexing.lexeme_start lexbuf in
	Buffer.clear string_buffer;
	let eend = string lexbuf in
        let loc = Location.find !Java_parser_base.cur_base start eend in
	STRING(loc, Buffer.contents string_buffer) )
# 706 "java_lexer.ml"

  | 54 ->
# 305 "java_lexer.mll"
      ( raise (Lexical_error (Lexing.lexeme lexbuf)) )
# 711 "java_lexer.ml"

  | 55 ->
# 307 "java_lexer.mll"
      ( EOF )
# 716 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf n

and string lexbuf =
    __ocaml_lex_string_rec lexbuf 76
and __ocaml_lex_string_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 311 "java_lexer.mll"
      ( Lexing.lexeme_end lexbuf )
# 727 "java_lexer.ml"

  | 1 ->
# 313 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	string lexbuf )
# 733 "java_lexer.ml"

  | 2 ->
# 316 "java_lexer.mll"
      ( raise (Lexical_error "unknown escape sequence in string") )
# 738 "java_lexer.ml"

  | 3 ->
# 318 "java_lexer.mll"
      ( (* no \n anymore in strings since java 1.4 *)
	raise (Lexical_error "string not terminated");
	Java_parser_base.next_line lexbuf; 
	Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	string lexbuf )
# 747 "java_lexer.ml"

  | 4 ->
# 324 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	string lexbuf )
# 753 "java_lexer.ml"

  | 5 ->
# 327 "java_lexer.mll"
      ( raise (Lexical_error "string not terminated") )
# 758 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec lexbuf n

and comment lexbuf =
    __ocaml_lex_comment_rec lexbuf 82
and __ocaml_lex_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 331 "java_lexer.mll"
      ( () )
# 769 "java_lexer.ml"

  | 1 ->
# 333 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf; comment lexbuf )
# 774 "java_lexer.ml"

  | 2 ->
# 335 "java_lexer.mll"
      ( comment lexbuf )
# 779 "java_lexer.ml"

  | 3 ->
# 337 "java_lexer.mll"
      ( comment lexbuf )
# 784 "java_lexer.ml"

  | 4 ->
# 339 "java_lexer.mll"
      ( raise (Lexical_error "comment not terminated") )
# 789 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec lexbuf n

and doc_comment lexbuf =
    __ocaml_lex_doc_comment_rec lexbuf 85
and __ocaml_lex_doc_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 343 "java_lexer.mll"
      ( Buffer.contents string_buffer )
# 800 "java_lexer.ml"

  | 1 ->
# 345 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf;  
	Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	doc_comment lexbuf )
# 807 "java_lexer.ml"

  | 2 ->
# 349 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	doc_comment lexbuf )
# 813 "java_lexer.ml"

  | 3 ->
# 352 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	doc_comment lexbuf )
# 819 "java_lexer.ml"

  | 4 ->
# 355 "java_lexer.mll"
      ( raise (Lexical_error "comment not terminated") )
# 824 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_doc_comment_rec lexbuf n

and comment_in_jml lexbuf =
    __ocaml_lex_comment_in_jml_rec lexbuf 88
and __ocaml_lex_comment_in_jml_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 359 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf) )
# 835 "java_lexer.ml"

  | 1 ->
# 361 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf; 
	Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	comment_in_jml lexbuf )
# 842 "java_lexer.ml"

  | 2 ->
# 365 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	comment_in_jml lexbuf )
# 848 "java_lexer.ml"

  | 3 ->
# 368 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	comment_in_jml lexbuf )
# 854 "java_lexer.ml"

  | 4 ->
# 371 "java_lexer.mll"
      ( raise (Lexical_error "jml spec not terminated") )
# 859 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_in_jml_rec lexbuf n

and jml_spec lexbuf =
    __ocaml_lex_jml_spec_rec lexbuf 91
and __ocaml_lex_jml_spec_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 375 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	comment_in_jml lexbuf; 
	jml_spec lexbuf )
# 872 "java_lexer.ml"

  | 1 ->
# 379 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf; 
	Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	jml_spec lexbuf )
# 879 "java_lexer.ml"

  | 2 ->
# 383 "java_lexer.mll"
      ( Buffer.contents string_buffer )
# 884 "java_lexer.ml"

  | 3 ->
# 385 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf;  
	Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	jml_spec lexbuf )
# 891 "java_lexer.ml"

  | 4 ->
# 389 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	jml_spec lexbuf )
# 897 "java_lexer.ml"

  | 5 ->
# 392 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	jml_spec lexbuf )
# 903 "java_lexer.ml"

  | 6 ->
# 395 "java_lexer.mll"
      ( raise (Lexical_error "jml spec not terminated") )
# 908 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_jml_spec_rec lexbuf n

and jml_line_spec lexbuf =
    __ocaml_lex_jml_line_spec_rec lexbuf 97
and __ocaml_lex_jml_line_spec_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 399 "java_lexer.mll"
      ( Java_parser_base.next_line lexbuf;  
	Buffer.contents string_buffer)
# 920 "java_lexer.ml"

  | 1 ->
# 402 "java_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	jml_line_spec lexbuf )
# 926 "java_lexer.ml"

  | 2 ->
# 410 "java_lexer.mll"
      ( raise (Lexical_error "jml spec not terminated") )
# 931 "java_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_jml_line_spec_rec lexbuf n

;;

