# 12 "jml_lexer.mll"
 

  open Jml_parser

  exception Lexical_error of string

  let string_buffer = Buffer.create 17

  let kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ "also", ALSO;
	  "and", AND;
	  "assert", ASSERT ;
	  "assignable", ASSIGNABLE;
	  "behavior", BEHAVIOR;
	  "byte", BYTE;
	  "decreases", DECREASES;
	  "diverges", DIVERGES;
	  "double", DOUBLE;
	  "ensures", ENSURES;
	  "ensures_redundantly", ENSURES;
	  "exceptional_behavior", EXCEPTIONAL_BEHAVIOR;
	  "false", FALSE(Location.dummy);
	  "final", FINAL;
	  "float", FLOAT;
	  "forall", FORALL;
	  "ghost", GHOST;
	  "if", IF;
	  "instanceof", INSTANCEOF;
	  "int", INT;
	  "invariant", INVARIANT;
	  "long", LONG;
	  "loop_invariant", LOOP_INVARIANT;
	  "measured_by", MEASURED_BY;
	  "model", MODEL;
	  "modifiable", ASSIGNABLE;
	  "modifies", ASSIGNABLE;
	  "new", NEW(Location.dummy);
	  "normal_behavior", NORMAL_BEHAVIOR;
	  "null", NULL(Location.dummy);
	  "old", OLD;
	  "private", PRIVATE;	
	  "protected", PROTECTED;
	  "public", PUBLIC;	
	  "pure", PURE;
	  "represents", REPRESENTS;
	  "requires", REQUIRES; 
	  "short", SHORT;
	  "signals", SIGNALS;
	  "spec_public", SPEC_PUBLIC;
	  "static", STATIC;
	  "super", SUPER(Location.dummy);
	  "true", TRUE(Location.dummy);
	  "when", WHEN;
	]
    in table


  let special_kw_table = 
    let table = Hashtbl.create 17 in
    let _ = 
      List.iter
	(fun (s,t) -> Hashtbl.add table s t)
	[ "everything", BSEVERYTHING ;
	  "exists", BSEXISTS ;
	  "fresh", BSFRESH(Location.dummy) ;
	  "forall", BSFORALL ;
	  "nothing", BSNOTHING;
	  "fields_of", BSFIELDSOF;
	  "old", BSOLD(Location.dummy) ;
	  "result", BSRESULT(Location.dummy) ;
	]
    in table

  let find_loc lexbuf =
    Location.find 
      !Jml_parser_base.cur_base 
      (Lexing.lexeme_start lexbuf) 
      (Lexing.lexeme_end lexbuf) 
      
  let id_or_kw lexbuf s =
    let l = find_loc lexbuf in
    try
      let k = Hashtbl.find kw_table s in
      (*i
	prerr_string "Keyword ";
	prerr_endline s;
	i*)
      begin
	match k with
	  | FALSE _ -> FALSE(l)
	  | NEW _ -> NEW(l)
	  | NULL _ -> NULL(l)
	  | SUPER _ -> SUPER(l)
	  | THIS _ -> THIS(l)
	  | TRUE _ -> TRUE(l)
	  | _ -> k
      end
    with
	Not_found ->
	  (*i
	    prerr_string "Ident ";
	    prerr_endline s;
	    i*)
	  ID(l,s)

  let special_id lexbuf s =
    let l = find_loc lexbuf in
   try
      let k = Hashtbl.find special_kw_table s in
      (*i
	prerr_string "Special Keyword ";
	prerr_endline s;
	i*)
      begin
	match k with
	  | BSRESULT _ -> BSRESULT(l)
	  | BSOLD _ -> BSOLD(l)
	  | _ -> k
      end
    with
	Not_found ->
	  (*i
	    prerr_string "Ident ";
	    prerr_endline s;
	    i*)
	  raise (Lexical_error ("unknown special JML keyword \\"^s)) 


   let remove_first_char s = String.sub s 1 (pred (String.length s))
   let remove_last_char s = String.sub s 0 (pred (String.length s))
   let remove_last_two_chars s = String.sub s 0 (pred (pred (String.length s)))

   exception Dotdot of Location.t * int


# 142 "jml_lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\198\255\003\000\254\255\062\000\160\000\249\255\248\255\
    \245\255\244\255\243\255\111\000\241\255\058\000\230\255\229\255\
    \228\255\227\255\140\000\093\000\095\000\125\000\141\000\237\000\
    \010\001\028\000\208\255\207\255\205\255\064\000\078\000\202\255\
    \201\255\200\255\199\255\197\255\237\255\203\255\204\255\160\000\
    \209\255\222\255\071\001\051\001\211\255\096\001\131\001\160\001\
    \195\001\223\001\216\255\210\255\247\000\233\001\220\255\020\001\
    \000\002\212\255\038\002\218\255\246\255\073\002\102\002\214\255\
    \106\001\010\002\224\255\142\000\235\255\143\000\225\255\226\255\
    \236\255\190\000\144\000\146\000\156\000\232\255\233\255\238\255\
    \253\255\242\000\252\255\112\002\219\000\255\255\109\000\075\001\
    \042\001\173\000\043\001\251\255";
  Lexing.lex_backtrk = 
   "\255\255\255\255\000\000\255\255\057\000\005\000\255\255\255\255\
    \255\255\255\255\255\255\013\000\255\255\015\000\255\255\255\255\
    \255\255\255\255\030\000\030\000\057\000\024\000\008\000\032\000\
    \032\000\057\000\255\255\255\255\255\255\049\000\016\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\034\000\038\000\
    \038\000\255\255\255\255\255\255\255\255\038\000\255\255\255\255\
    \042\000\255\255\036\000\255\255\255\255\040\000\255\255\255\255\
    \255\255\040\000\255\255\031\000\255\255\029\000\255\255\255\255\
    \255\255\030\000\021\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\004\000\255\255\255\255\255\255\002\000\
    \255\255\003\000\002\000\255\255";
  Lexing.lex_default = 
   "\001\000\000\000\255\255\000\000\255\255\255\255\000\000\000\000\
    \000\000\000\000\000\000\255\255\000\000\255\255\000\000\000\000\
    \000\000\000\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\039\000\000\000\000\000\000\000\255\255\255\255\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\255\255\
    \000\000\000\000\255\255\255\255\000\000\255\255\255\255\255\255\
    \255\255\255\255\000\000\000\000\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\000\000\000\000\255\255\255\255\000\000\
    \255\255\255\255\000\000\255\255\000\000\255\255\000\000\000\000\
    \000\000\255\255\255\255\255\255\255\255\000\000\000\000\000\000\
    \000\000\081\000\000\000\255\255\087\000\000\000\255\255\087\000\
    \090\000\255\255\090\000\000\000";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\002\000\003\000\000\000\002\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \002\000\021\000\033\000\002\000\000\000\012\000\013\000\025\000\
    \026\000\027\000\010\000\008\000\007\000\009\000\022\000\011\000\
    \024\000\023\000\023\000\023\000\023\000\023\000\023\000\023\000\
    \023\000\023\000\017\000\006\000\018\000\020\000\019\000\016\000\
    \034\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\031\000\004\000\032\000\015\000\005\000\
    \079\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\029\000\030\000\028\000\014\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\080\000\070\000\069\000\067\000\083\000\081\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\072\000\066\000\060\000\038\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\061\000\061\000\061\000\061\000\040\000\
    \071\000\073\000\036\000\037\000\068\000\071\000\078\000\076\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\077\000\003\000\085\000\000\000\000\000\075\000\
    \000\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\074\000\082\000\085\000\000\000\005\000\
    \035\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\047\000\255\255\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\044\000\043\000\044\000\003\000\255\255\000\000\086\000\
    \047\000\041\000\046\000\046\000\046\000\046\000\046\000\046\000\
    \046\000\046\000\045\000\045\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\044\000\043\000\
    \044\000\044\000\043\000\044\000\089\000\255\255\041\000\000\000\
    \000\000\041\000\000\000\000\000\000\000\000\000\055\000\000\000\
    \055\000\000\000\042\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\056\000\056\000\056\000\255\255\044\000\043\000\
    \044\000\000\000\000\000\000\000\000\000\000\000\041\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\000\000\042\000\000\000\000\000\000\000\000\000\000\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\047\000\000\000\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\065\000\065\000\065\000\065\000\065\000\065\000\
    \065\000\065\000\065\000\065\000\044\000\043\000\044\000\255\255\
    \058\000\058\000\058\000\058\000\058\000\058\000\000\000\000\000\
    \000\000\047\000\000\000\046\000\046\000\046\000\046\000\046\000\
    \046\000\046\000\046\000\045\000\045\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\044\000\043\000\044\000\044\000\
    \043\000\044\000\000\000\000\000\000\000\000\000\051\000\054\000\
    \048\000\048\000\048\000\048\000\048\000\048\000\048\000\048\000\
    \048\000\048\000\000\000\082\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\050\000\049\000\050\000\044\000\
    \043\000\044\000\000\000\000\000\000\000\000\000\000\000\054\000\
    \000\000\000\000\255\255\048\000\048\000\048\000\048\000\048\000\
    \048\000\048\000\048\000\048\000\048\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\050\000\049\000\050\000\050\000\
    \049\000\050\000\052\000\000\000\052\000\000\000\000\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\000\000\000\000\000\000\000\000\050\000\
    \049\000\050\000\091\000\255\255\000\000\050\000\000\000\050\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\065\000\065\000\065\000\065\000\065\000\065\000\
    \065\000\065\000\065\000\065\000\057\000\000\000\057\000\000\000\
    \000\000\000\000\000\000\255\255\000\000\050\000\063\000\050\000\
    \063\000\000\000\000\000\000\000\000\000\000\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \000\000\000\000\000\000\000\000\057\000\000\000\057\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\000\000\063\000\000\000\
    \063\000\000\000\059\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\000\000\000\000\000\000\000\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\063\000\062\000\063\000\
    \000\000\064\000\059\000\064\000\000\000\000\000\065\000\065\000\
    \065\000\065\000\065\000\065\000\065\000\065\000\065\000\065\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\000\000\000\000\000\000\063\000\062\000\063\000\
    \000\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\000\000\000\000\000\000\000\000\083\000\
    \000\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\255\255\002\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\002\000\255\255\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \013\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\011\000\019\000\019\000\020\000\004\000\011\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\018\000\021\000\022\000\029\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\039\000\
    \018\000\018\000\030\000\030\000\067\000\069\000\074\000\075\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\076\000\086\000\089\000\255\255\255\255\073\000\
    \255\255\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\073\000\081\000\084\000\255\255\005\000\
    \000\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\005\000\005\000\005\000\005\000\005\000\
    \005\000\005\000\005\000\023\000\025\000\023\000\023\000\023\000\
    \023\000\023\000\023\000\023\000\023\000\023\000\023\000\052\000\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\023\000\023\000\023\000\088\000\090\000\255\255\084\000\
    \024\000\023\000\024\000\024\000\024\000\024\000\024\000\024\000\
    \024\000\024\000\024\000\024\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\024\000\024\000\
    \024\000\023\000\023\000\023\000\088\000\090\000\024\000\255\255\
    \255\255\023\000\255\255\255\255\255\255\255\255\043\000\255\255\
    \043\000\255\255\024\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\087\000\024\000\024\000\
    \024\000\255\255\255\255\255\255\255\255\255\255\024\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\255\255\024\000\255\255\255\255\255\255\255\255\255\255\
    \042\000\042\000\042\000\042\000\042\000\042\000\045\000\255\255\
    \045\000\045\000\045\000\045\000\045\000\045\000\045\000\045\000\
    \045\000\045\000\064\000\064\000\064\000\064\000\064\000\064\000\
    \064\000\064\000\064\000\064\000\045\000\045\000\045\000\087\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\255\255\255\255\
    \255\255\046\000\255\255\046\000\046\000\046\000\046\000\046\000\
    \046\000\046\000\046\000\046\000\046\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\045\000\045\000\045\000\046\000\
    \046\000\046\000\255\255\255\255\255\255\255\255\047\000\046\000\
    \047\000\047\000\047\000\047\000\047\000\047\000\047\000\047\000\
    \047\000\047\000\255\255\084\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\047\000\047\000\047\000\046\000\
    \046\000\046\000\255\255\255\255\255\255\255\255\255\255\046\000\
    \255\255\255\255\081\000\048\000\048\000\048\000\048\000\048\000\
    \048\000\048\000\048\000\048\000\048\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\047\000\047\000\047\000\048\000\
    \048\000\048\000\049\000\255\255\049\000\255\255\255\255\049\000\
    \049\000\049\000\049\000\049\000\049\000\049\000\049\000\049\000\
    \049\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\255\255\255\255\255\255\255\255\048\000\
    \048\000\048\000\088\000\090\000\255\255\053\000\255\255\053\000\
    \056\000\056\000\056\000\056\000\056\000\056\000\056\000\056\000\
    \056\000\056\000\065\000\065\000\065\000\065\000\065\000\065\000\
    \065\000\065\000\065\000\065\000\056\000\255\255\056\000\255\255\
    \255\255\255\255\255\255\087\000\255\255\053\000\065\000\053\000\
    \065\000\255\255\255\255\255\255\255\255\255\255\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \255\255\255\255\255\255\255\255\056\000\255\255\056\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\255\255\065\000\255\255\
    \065\000\255\255\058\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\061\000\061\000\061\000\061\000\061\000\061\000\061\000\
    \061\000\061\000\061\000\255\255\255\255\255\255\255\255\058\000\
    \058\000\058\000\058\000\058\000\058\000\061\000\061\000\061\000\
    \255\255\062\000\058\000\062\000\255\255\255\255\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\255\255\255\255\255\255\061\000\061\000\061\000\
    \255\255\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\255\255\255\255\255\255\255\255\083\000\
    \255\255\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\083\000\083\000\083\000\083\000\083\000\
    \083\000\083\000\083\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_base_code = 
   "";
  Lexing.lex_backtrk_code = 
   "";
  Lexing.lex_default_code = 
   "";
  Lexing.lex_trans_code = 
   "";
  Lexing.lex_check_code = 
   "";
  Lexing.lex_code = 
   "";
}

let rec token lexbuf =
    __ocaml_lex_token_rec lexbuf 0
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 154 "jml_lexer.mll"
      ( token lexbuf )
# 428 "jml_lexer.ml"

  | 1 ->
# 156 "jml_lexer.mll"
      ( Jml_parser_base.next_line lexbuf; token lexbuf )
# 433 "jml_lexer.ml"

  | 2 ->
# 158 "jml_lexer.mll"
      ( comment lexbuf )
# 438 "jml_lexer.ml"

  | 3 ->
# 160 "jml_lexer.mll"
      ( Jml_parser_base.next_line lexbuf; token lexbuf )
# 443 "jml_lexer.ml"

  | 4 ->
# 162 "jml_lexer.mll"
      ( special_id lexbuf (remove_first_char (Lexing.lexeme lexbuf)) )
# 448 "jml_lexer.ml"

  | 5 ->
# 164 "jml_lexer.mll"
      ( id_or_kw lexbuf (Lexing.lexeme lexbuf) )
# 453 "jml_lexer.ml"

  | 6 ->
# 166 "jml_lexer.mll"
      ( SEMICOLON )
# 458 "jml_lexer.ml"

  | 7 ->
# 168 "jml_lexer.mll"
      ( COMMA )
# 463 "jml_lexer.ml"

  | 8 ->
# 170 "jml_lexer.mll"
      ( DOT )
# 468 "jml_lexer.ml"

  | 9 ->
# 172 "jml_lexer.mll"
      ( DOTDOT )
# 473 "jml_lexer.ml"

  | 10 ->
# 174 "jml_lexer.mll"
      ( PLUS(find_loc lexbuf) )
# 478 "jml_lexer.ml"

  | 11 ->
# 176 "jml_lexer.mll"
      ( MINUS(find_loc lexbuf) )
# 483 "jml_lexer.ml"

  | 12 ->
# 178 "jml_lexer.mll"
      ( STAR )
# 488 "jml_lexer.ml"

  | 13 ->
# 180 "jml_lexer.mll"
      ( SLASH )
# 493 "jml_lexer.ml"

  | 14 ->
# 182 "jml_lexer.mll"
      ( PERCENT )
# 498 "jml_lexer.ml"

  | 15 ->
# 184 "jml_lexer.mll"
      ( AMPERSAND )
# 503 "jml_lexer.ml"

  | 16 ->
# 186 "jml_lexer.mll"
      ( VERTICALBAR )
# 508 "jml_lexer.ml"

  | 17 ->
# 188 "jml_lexer.mll"
      ( AMPERSANDAMPERSAND )
# 513 "jml_lexer.ml"

  | 18 ->
# 190 "jml_lexer.mll"
      ( VERTICALBARVERTICALBAR )
# 518 "jml_lexer.ml"

  | 19 ->
# 192 "jml_lexer.mll"
      ( LEFTARROW )
# 523 "jml_lexer.ml"

  | 20 ->
# 194 "jml_lexer.mll"
      ( EQEQGT )
# 528 "jml_lexer.ml"

  | 21 ->
# 196 "jml_lexer.mll"
      ( LTEQEQ  )
# 533 "jml_lexer.ml"

  | 22 ->
# 198 "jml_lexer.mll"
      ( LTEQEQGT )
# 538 "jml_lexer.ml"

  | 23 ->
# 200 "jml_lexer.mll"
      ( LTEQBANGEQGT )
# 543 "jml_lexer.ml"

  | 24 ->
# 202 "jml_lexer.mll"
      ( BANG(find_loc lexbuf) )
# 548 "jml_lexer.ml"

  | 25 ->
# 204 "jml_lexer.mll"
      ( TILDA(find_loc lexbuf) )
# 553 "jml_lexer.ml"

  | 26 ->
# 206 "jml_lexer.mll"
      ( CARET )
# 558 "jml_lexer.ml"

  | 27 ->
# 208 "jml_lexer.mll"
      ( QUESTIONMARK )
# 563 "jml_lexer.ml"

  | 28 ->
# 210 "jml_lexer.mll"
      ( COLON )
# 568 "jml_lexer.ml"

  | 29 ->
# 212 "jml_lexer.mll"
      ( SHIFT(Lexing.lexeme lexbuf) )
# 573 "jml_lexer.ml"

  | 30 ->
# 214 "jml_lexer.mll"
      ( COMP(Lexing.lexeme lexbuf) )
# 578 "jml_lexer.ml"

  | 31 ->
# 216 "jml_lexer.mll"
      ( EQOP(Lexing.lexeme lexbuf) )
# 583 "jml_lexer.ml"

  | 32 ->
# 221 "jml_lexer.mll"
      ( INTEGER(find_loc lexbuf,int_of_string (Lexing.lexeme lexbuf)) )
# 588 "jml_lexer.ml"

  | 33 ->
# 223 "jml_lexer.mll"
      ( INTEGER 
	  (find_loc lexbuf,int_of_string (remove_last_char (Lexing.lexeme lexbuf))) )
# 594 "jml_lexer.ml"

  | 34 ->
# 229 "jml_lexer.mll"
      ( INTEGER (find_loc lexbuf,int_of_string ("0o" ^ (Lexing.lexeme lexbuf))) )
# 599 "jml_lexer.ml"

  | 35 ->
# 231 "jml_lexer.mll"
      ( INTEGER 
	  (find_loc lexbuf,int_of_string 
	     ("0o" ^ (remove_last_char (Lexing.lexeme lexbuf)))) )
# 606 "jml_lexer.ml"

  | 36 ->
# 238 "jml_lexer.mll"
    ( INTEGER (find_loc lexbuf,int_of_string (Lexing.lexeme lexbuf)) )
# 611 "jml_lexer.ml"

  | 37 ->
# 240 "jml_lexer.mll"
    ( INTEGER (find_loc lexbuf,int_of_string (remove_last_char (Lexing.lexeme lexbuf))) )
# 616 "jml_lexer.ml"

  | 38 ->
# 245 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string  (Lexing.lexeme lexbuf)) )
# 621 "jml_lexer.ml"

  | 39 ->
# 247 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string  (remove_last_char (Lexing.lexeme lexbuf))) )
# 626 "jml_lexer.ml"

  | 40 ->
# 249 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string  (Lexing.lexeme lexbuf)) )
# 631 "jml_lexer.ml"

  | 41 ->
# 251 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string  (remove_last_char (Lexing.lexeme lexbuf))) )
# 636 "jml_lexer.ml"

  | 42 ->
# 253 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string  (Lexing.lexeme lexbuf)) )
# 641 "jml_lexer.ml"

  | 43 ->
# 255 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string  (remove_last_char (Lexing.lexeme lexbuf))) )
# 646 "jml_lexer.ml"

  | 44 ->
# 257 "jml_lexer.mll"
      ( REAL (find_loc lexbuf,float_of_string (remove_last_char (Lexing.lexeme lexbuf))) )
# 651 "jml_lexer.ml"

  | 45 ->
# 262 "jml_lexer.mll"
      ( raise 
	  (Dotdot
	     (find_loc lexbuf,int_of_string 
		(remove_last_two_chars (Lexing.lexeme lexbuf)))) )
# 659 "jml_lexer.ml"

  | 46 ->
# 270 "jml_lexer.mll"
      ( CHARACTER (find_loc lexbuf,Lexing.lexeme_char lexbuf 1) )
# 664 "jml_lexer.ml"

  | 47 ->
# 273 "jml_lexer.mll"
      ( LEFTPAR(find_loc lexbuf) )
# 669 "jml_lexer.ml"

  | 48 ->
# 275 "jml_lexer.mll"
      ( RIGHTPAR )
# 674 "jml_lexer.ml"

  | 49 ->
# 277 "jml_lexer.mll"
      ( LEFTBRACE )
# 679 "jml_lexer.ml"

  | 50 ->
# 279 "jml_lexer.mll"
      ( RIGHTBRACE )
# 684 "jml_lexer.ml"

  | 51 ->
# 281 "jml_lexer.mll"
      ( LEFTBRACEBAR )
# 689 "jml_lexer.ml"

  | 52 ->
# 283 "jml_lexer.mll"
      ( BARRIGHTBRACE )
# 694 "jml_lexer.ml"

  | 53 ->
# 285 "jml_lexer.mll"
      ( LEFTBRACKET )
# 699 "jml_lexer.ml"

  | 54 ->
# 287 "jml_lexer.mll"
      ( RIGHTBRACKET )
# 704 "jml_lexer.ml"

  | 55 ->
# 289 "jml_lexer.mll"
      ( let start = Lexing.lexeme_start lexbuf in
	Buffer.clear string_buffer;
	let l = string lexbuf in
	let eend = Lexing.lexeme_end lexbuf in
	STRING(Location.find !Jml_parser_base.cur_base start eend,
	       Buffer.contents string_buffer) )
# 714 "jml_lexer.ml"

  | 56 ->
# 296 "jml_lexer.mll"
      ( token lexbuf )
# 719 "jml_lexer.ml"

  | 57 ->
# 298 "jml_lexer.mll"
      ( raise (Lexical_error (Lexing.lexeme lexbuf)) )
# 724 "jml_lexer.ml"

  | 58 ->
# 300 "jml_lexer.mll"
      ( EOF )
# 729 "jml_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_token_rec lexbuf n

and string lexbuf =
    __ocaml_lex_string_rec lexbuf 84
and __ocaml_lex_string_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 304 "jml_lexer.mll"
      ( find_loc lexbuf )
# 740 "jml_lexer.ml"

  | 1 ->
# 306 "jml_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	string lexbuf )
# 746 "jml_lexer.ml"

  | 2 ->
# 309 "jml_lexer.mll"
      ( Buffer.add_string string_buffer (Lexing.lexeme lexbuf);
	string lexbuf )
# 752 "jml_lexer.ml"

  | 3 ->
# 312 "jml_lexer.mll"
      ( raise (Lexical_error "string not terminated") )
# 757 "jml_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_string_rec lexbuf n

and comment lexbuf =
    __ocaml_lex_comment_rec lexbuf 88
and __ocaml_lex_comment_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 316 "jml_lexer.mll"
      ( token lexbuf )
# 768 "jml_lexer.ml"

  | 1 ->
# 318 "jml_lexer.mll"
      ( Jml_parser_base.next_line lexbuf; comment lexbuf )
# 773 "jml_lexer.ml"

  | 2 ->
# 320 "jml_lexer.mll"
      ( comment lexbuf )
# 778 "jml_lexer.ml"

  | 3 ->
# 322 "jml_lexer.mll"
      ( comment lexbuf )
# 783 "jml_lexer.ml"

  | 4 ->
# 324 "jml_lexer.mll"
      ( raise (Lexical_error "comment not terminated") )
# 788 "jml_lexer.ml"

  | n -> lexbuf.Lexing.refill_buff lexbuf; __ocaml_lex_comment_rec lexbuf n

;;

# 328 "jml_lexer.mll"
 

let dotdot_mem = ref false
 
let next_token lexbuf =
  if !dotdot_mem then
    begin
      dotdot_mem := false;
      DOTDOT
    end
  else
    begin
      try
	token lexbuf
      with
	Dotdot(l,n) ->
	  dotdot_mem := true;
	  INTEGER(l,n)
    end
	      

# 816 "jml_lexer.ml"
