
/* Dijkstra's dutch flag */

package flag;

public class Flag {
    
    public static final int BLUE = 1, WHITE = 2, RED = 3;
    
    /*@ private normal_behavior
      @   modifiable \nothing;
      @   ensures \result <==> (i == BLUE || i == WHITE || i == RED);
      @*/
    private static /*@ pure @*/ boolean isColor(int i);

    public int t[];

    /*@ public invariant
      @  t != null && (\forall int k; 0 <= k && k < t.length; isColor(t[k]));
      @*/

    /*@ private normal_behavior
      @   requires 0 <= i && i <= j && j <= t.length ;
      @   modifiable \nothing;
      @   ensures \result <==> (\forall int k; i <= k && k < j; t[k] == c);
      @*/
    private /*@ pure @*/ boolean isMonochrome(int i, int j, int c);

    /*@ private normal_behavior
      @   requires 0 <= i && i < t.length && 0 <= j && j < t.length;
      @   modifiable t[i],t[j];
      @   ensures 	  
      @     t[i] == \old(t[j]) && t[j] == \old(t[i]);
      @*/
    private void swap(int i, int j);

    /*@ public normal_behavior
      @   //modifiable t[*];
      @   ensures 
      @     (\exists int b,r; isMonochrome(0,b,BLUE) &&
      @                       isMonochrome(b,r,WHITE) &&
      @                       isMonochrome(r,t.length,RED));
      @*/
    public void flag() {
	int b = 0;
	int i = 0;
	int r = t.length;
	/*@ loop_invariant
	  @   (\forall int k; 0 <= k && k < t.length; isColor(t[k])) &&
	  @   0 <= b && b <= i && i <= r && r <= t.length &&
	  @   isMonochrome(0,b,BLUE) &&
	  @   isMonochrome(b,i,WHITE) &&
          @   isMonochrome(r,t.length,RED);
	  @ decreases r - i; 
	  @*/
	while (i < r) {
	    switch (t[i]) {
	    case BLUE:  
		swap(b, i);
		b++;
		i++;
		break;	    
	    case WHITE: 
		i++; 
		break;
	    case RED: 
		r--;
		swap(r, i);
		break;
	    }
	}
    }
}
